# friendly-date
# Author: Abizer Kapadia
# Date: Sat Mar 17 17:29:21 2012

window.FriendlyDate = class FriendlyDate
  constructor: (@friendly_date) ->
    
    [
      all, @date, @time
    ] = @parseDateTime @friendly_date
    
    @date_length = @date.length
    if @date_length > 9 # Not today, tomorrow or yesterday
      [
        all, @verbose_day, @day, @verbose_month 
        @year
      ] = @parseDate @date
    
    if ":" in @time
      [
       all, @hour, @minutes, @period

      ] = @parseTime @time
      @time = (new FormatJSTime @hour, @minutes, @period).toJS()
    else
      [
        all, @hour, @period
 
      ] = @parseShortTime @time
      @time = (new FormatJSTime @hour, @period).toJS()
    
    
  parseDateTime: (friendlyDate) ->
    result = friendlyDate.match /(.*)at(.*)/ # Date at Time
    r.trim() for r in result
    
  parseDate: (date) ->
    pattern = ///
      (\w+)           # Verbose Day
      \sthe\s         
      (\d{1,2})       # Day
      \w+\sof\s
      (\w+)           # Month
      \s
      (\d{4})         # Year
    ///
    result = date.match pattern
    r.trim() for r in result
  
  parseTime: (time) ->
    pattern = ///
      (\d{1,2}):      # Hour
      (\d{2})         # Minutes
      (\w{2})         # Period
    ///
    result = time.match pattern
    r.trim() for r in result
  
  parseShortTime: (time) ->
    pattern = ///
      (\d{1,2})       # Hour
      (\w{2})         # Period
    ///
    result = time.match pattern
    r.trim() for r in result
    
  toJS: ->
    pattern = ///
      (.*)                  # Date
      \d{2}:\d{2}:\d{2}.*
    ///
    
    switch @date_length
      when 5 # Today
        date = new Date()
        [all, date] = date.toString().match pattern
        "#{ date } #{ @time }"
      when 8 # Tomorrow
        date = new Date()
        date = new Date(date.getTime() + 86400000)
        [all, date] = date.toString().match pattern
        "#{ date } #{ @time }"
      when 9 # Yesterday
        date = new Date()
        date = new Date(date.getTime() - 86400000)
        [all, date] = date.toString().match pattern
        "#{ date } #{ @time }"
      else
        "#{ @verbose_day.substr(0, 3) } #{ @verbose_month.substr(0, 3) } #{ @day } #{ @year } #{ @time }"
      
  toJSON: -> { @friendly_date, js_date: @toJS() }
    
    
window.StdDate = class StdDate
  constructor: (rawDateTime) ->
    [
      all, @verbose_day, @verbose_month
      @day, @year, @hour, @minutes
      
    ] = @parseRawDate rawDateTime.toString()
    @month = (new FormatMonth @verbose_month).toIndex()
    @time = (new FormatTime @hour, @minutes).toFriendly()
    @date = new Date @year, @month, @day
    @date = new Friendly @date
    
  parseRawDate: (rawDateTime="") ->
    pattern = ///
      (\w{3})         # Day 
      \s
      (\w{3})         # Month
      \s
      (\d{2})         # DayDate
      \s
      (\d{4})         # Year
      \s
      (\d{2})
      :        # Hour
      (\d{2})         # Minutes
    ///
    result = rawDateTime.match pattern
    r.trim() for r in result
    
  toFriendly: -> "#{@date.toFriendly()} at #{@time}"
  
  toJSON: ->  { 
                friendly_date: @toFriendly(), 
                js_date: "#{ @verbose_day } #{ @verbose_month } #{ @day } #{ @year } #{ @hour }:#{ @minutes }:00" 
              }
    
window.FormatMonth = class FormatMonth
  constructor: (@verbose_month) ->
    
  toIndex: ->
    months = [
      "Jan"
      "Feb"
      "Mar"
      "Apr"
      "May"
      "Jun"
      "Jul"
      "Aug"
      "Sep"
      "Oct"
      "Nov"
      "Dec"
    ]
    months.indexOf @verbose_month
  
window.FormatTime = class FormatTime
  constructor: (@hour, @minutes) ->
    
  toFriendly: ->
    hours = {
      '00': "12", '01': "1", '02': "2", '03': "3"
      '04': "4", '05': "5", '06': "6", '07': "7"
      '08': "8", '09': "9", '10': "10", '11': "11"
      '12': "12", '13': "1", '14': "2", '15': "3"
      '16': "4", '17': "5", '18': "6", '19': "7"
      '20': "8", '21': "9", '22': "10", '23': "11"
    }
    am = ['00','01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11' ]
    minutes = [0..9]
    period = if @hour in am then "am" else "pm"
    @minutes = if @minutes in minutes then "0#{ @minutes }" else @minutes   
    if @minutes is "00"
      "#{ hours[@hour] }#{ period }"
    else
      "#{ hours[@hour] }:#{ @minutes }#{ period }"

window.FormatJSTime = class FormatJSTime
    constructor: (time...) ->
      if time.length is 3
        @hour = time[0]
        @minutes = time[1]
        @period = time[2]
      else
        @hour = time[0]
        @period = time[1]      
      
    toJS: ->
      hours = {
        "am": {
          "12": "00", "1": "01", "2", "02"
          "3": "03", "4": "04", "5": "05"
          "6": "06", "7": "07", "8": "08"
          "9": "09", "10": "10", "11": "11"
        }
        "pm": {
          "12": "12", "1": "13", "2": "14"
          "3": "15", "4": "16", "5": "17"
          "6": "18", "7": "19", "8": "20"
          "9": "21", "10": "22", "11": "23"
        }
      }
      if @minutes
        "#{ hours[@period][@hour] }:#{ @minutes }:00"
      else
        "#{ hours[@period][@hour] }:00:00"
    
window.Friendly = class Friendly
  constructor: (@date) ->
    @today = new Date()
  
  toFriendly: ->
    switch @date.toDateString()
      when @today.toDateString() then "Today"
      when (new Date(@today.getTime() + 86400000)).toDateString() then "Tomorrow"
      when (new Date(@today.getTime() - 86400000)).toDateString() then "Yesterday"
      else
        days = [
          "Sunday"
          "Monday"
          "Tuesday"
          "Wednesday"
          "Thursday"
          "Friday"
          "Saturday"
        ]
        friendlyDays = [
          "1st", "2nd", "3rd", "4th", "5th"
          "6th", "7th", "8th", "9th", "10th"
          "11th", "12th", "13th", "14th", "15th"
          "16th", "17th", "18th", "19th", "20th"
          "21st", "22nd", "23rd", "24th", "25th"
          "26th", "27th", "28th", "29th", "30th"
          "31st"
        ]
        months = [
          "January"
          "February"
          "March"
          "April"
          "May"
          "June"
          "July"
          "August"
          "September"
          "October"
          "November"
          "December"
        ]
      
        "#{days[@date.getDay()]} the #{friendlyDays[@date.getDate()-1]} of #{months[@date.getMonth()]} #{ @date.getFullYear() }"
          
    
    
    

    
