describe "Format Month", ->
  it "takes a 3 letter month and returns the correct JS index", ->
    @verbose_month = "Mar"
    @month = new FormatMonth @verbose_month
    (expect @month.toIndex()).toEqual 2
    
describe "Format Time", ->
  it "takes hours and minutes and returns a formated time", ->
    @time = new FormatTime 19, 30
    (expect @time.toFriendly()).toEqual "7:30pm"
  
  it "handles 00 minutes correctly", ->
    @time = new FormatTime '00', 00
    (expect @time.minutes).toEqual 00
  
  it "handles 00 hours correctly", ->
    @time = new FormatTime '00', 00
    (expect @time.hour).toEqual '00'
    
  it "handles 01 hours correctly", ->
    @time = new FormatTime '01', 00
    (expect @time.hour).toEqual '01'
    
  it "handles 00:00 correctly", ->
    @time = new FormatTime '00', 00
    (expect @time.toFriendly()).toEqual "12am"
    
  it "handles 01:00 correctly", ->
    @time = new FormatTime '01', 00
    (expect @time.toFriendly()).toEqual "1am"

describe "Friendly", ->
    
  it "knows it's today", ->
    @today = new Date()
    @date = new Friendly @today
    (expect @date.toFriendly()).toEqual "Today"
    
  it "knows it's yesterday", ->
    @today = new Date()
    @yesterday = new Date(@today.getTime() - 86400000)
    @date = new Friendly @yesterday
    (expect @date.toFriendly()).toEqual "Yesterday" 
    
  it "knows it's tomorrow", ->
    @today = new Date()
    @tomorrow = new Date(@today.getTime() + 86400000)
    @date = new Friendly @tomorrow
    (expect @date.toFriendly()).toEqual "Tomorrow"
    
  it "can friendly format a date far in the future", ->
     @distant_date = new Date(2020, 03, 30)
     @date = new Friendly @distant_date
     (expect @date.toFriendly()).toEqual "Thursday the 30th of April 2020"

describe "StdDate", ->

  beforeEach ->
    @date = new Date(2020, 02, 31, 19, 30)
    @date = new StdDate @date
  
  it "extracts day", ->
    (expect @date.day).toEqual '31'
  
  it "extracts month", ->
    (expect @date.month).toEqual 2
    
  it "extracts year", ->
    (expect @date.year).toEqual '2020'
    
  it "extracts time", ->
    (expect @date.time).toEqual '7:30pm'
    
  it "parses day, month and year into a JS date object", ->
    @our_date = new Date 2020, 02, 31
    (expect @date.date.date).toEqual @our_date
    
  it "returns a friendly version of the distant date", ->
    (expect @date.toFriendly()).toEqual "Tuesday the 31st of March 2020 at 7:30pm"
    
  it "returns both formats as JSON", ->
    (expect @date.toJSON()).toEqual { 
                                      friendly_date : 'Tuesday the 31st of March 2020 at 7:30pm'
                                      js_date : 'Tue Mar 31 2020 19:30:00' 
                                    }
  it "handles 00 hours correctly", ->
    @date = 'Thu Apr 30 2020 00:00:00 GMT+0100 (BST)'
    @date = new StdDate @date
    (expect @date.hour).toEqual '00'
  
  it "handles 00:00 correctly", ->
    @date = 'Thu Apr 30 2020 00:00:00 GMT+0100 (BST)'
    @date = new StdDate @date
    (expect @date.time).toEqual '12am'
    
  it "can go from stdDate to Friendly date and back to stdDate again", ->
      @js_date = 'Tue Mar 31 2020 19:30:00'
      @date = new StdDate 'Tue Mar 31 2020 19:30:00'
      @friendly_date = @date.toFriendly()
      @date = new FriendlyDate @friendly_date
      (expect @date.toJS()).toEqual @js_date
    
describe "FriendlyDate", ->
  beforeEach ->
    @date = "Tuesday the 31st of March 2020 at 7:30pm"
    @jsDate = "Tue Mar 31 2020 19:30:00"
    @date = new FriendlyDate @date
    @pattern = ///
      (.*)                  # Date
      (\d{2}:\d{2}:\d{2})   # Time
      .*
    ///
    
  it "splits up the date", ->
    (expect @date.date).toEqual "Tuesday the 31st of March 2020"
    
  it "extracts the verbose day", ->
      (expect @date.verbose_day).toEqual "Tuesday"
    
  it "converts hours, minutes and period into JSTime", ->
    (expect @date.time).toEqual "19:30:00"
  
  it "converts friendly date to JS date", ->
    (expect @date.toJS()).toEqual @jsDate
  
  it "returns both formats as JSON", ->
    (expect @date.toJSON()).toEqual { 
                                      friendly_date : 'Tuesday the 31st of March 2020 at 7:30pm'
                                      js_date : 'Tue Mar 31 2020 19:30:00' 
                                    }
  
  it "returns correct JS if an hour like 12am", ->
    @date = "Tuesday the 31st of March 2020 at 12am"
    @date = new FriendlyDate @date
    (expect @date.time).toEqual "00:00:00"
    
  it "returns the correct JS if today", ->
    @date = "Today at 12pm"
    @date = new FriendlyDate @date
    @today = new Date()
    @today.setHours(12,0,0,0)
    [all, date, time] = @today.toString().match @pattern
    @today = "#{ date } #{ time }"
    (expect @date.toJS()).toEqual @today
    
  it "returns the correct JS if yesterday", ->
    @date = "Yesterday at 12pm"
    @date = new FriendlyDate @date
    @today = new Date()
    @yesterday = new Date(@today.getTime() - 86400000)
    @yesterday.setHours(12,0,0,0)
    [all, date, time] = @yesterday.toString().match @pattern
    @yesterday = "#{ date } #{ time }"
    (expect @date.toJS()).toEqual @yesterday 
    
  it "returns the correct JS if tomorrow", ->
    @date = "Tomorrow at 12pm"
    @date = new FriendlyDate @date
    @today = new Date()
    @tomorrow = new Date(@today.getTime() + 86400000)
    @tomorrow.setHours(12,0,0,0)
    [all, date, time] = @tomorrow.toString().match @pattern
    @tomorrow = "#{ date } #{ time }"
    (expect @date.toJS()).toEqual @tomorrow
      
describe "JSTime", ->
  it "converts friendly time to std JS time", ->
    @time = new FormatJSTime "7", "30", "pm"
    (expect @time.toJS()).toEqual "19:30:00"
  
  it "converts 12am style times back to JS time", ->
    @time = new FormatJSTime "7", "pm"
    (expect @time.toJS()).toEqual "19:00:00"


    